from typing import Iterable, Union

from packaging.version import Version

import redgrease.runtime


class ReaderBase(redgrease.runtime.GearsBuilder):
    def __init__(
        self,
        *args,
        requirements: Iterable[str] = None,
        require_runtime: Union[Version, str, bool] = True,
        **kwargs,
    ):
        if require_runtime:
            if isinstance(require_runtime, Version):
                runtime_package = f"redgrease[runtime]=={require_runtime}"
            elif isinstance(require_runtime, str):
                if require_runtime[0] in ["=", ">", "<", "!"]:
                    runtime_package = f"redgrease[runtime]{require_runtime}"
                elif require_runtime[0].isnumeric():
                    runtime_package = f"redgrease[runtime]=={require_runtime}"
                else:
                    runtime_package = require_runtime
            else:
                runtime_package = "redgrease[runtime]"
            self.requirements = [runtime_package]
        else:
            self.requirements = []

        if requirements:
            self.requirements += list(requirements)

        super().__init__(*args, **kwargs)



from dataclasses import dataclass, field
from typing import Any, Dict, Optional

import redgrease.typing as optype
from redgrease.sugar import TriggerMode


# Redis Gears Operations
@dataclass
class Operation:
    pass


@dataclass
class Run(Operation):
    arg: Optional[str] = None
    convertToStr: bool = True
    collect: bool = True
    kargs: Dict[str, Any] = field(default_factory=dict)


@dataclass
class Register(Operation):
    prefix: str = "*"
    convertToStr: bool = True
    collect: bool = True
    mode: str = TriggerMode.Async
    onRegistered: Optional[optype.Callback] = None
    trigger: Optional[str] = None
    kargs: Dict[str, Any] = field(default_factory=dict)


@dataclass
class Map(Operation):
    op: optype.Mapper


@dataclass
class FlatMap(Operation):
    op: optype.Expander


@dataclass
class ForEach(Operation):
    op: optype.Processor


@dataclass
class Filter(Operation):
    op: optype.Filterer


@dataclass
class Accumulate(Operation):
    op: optype.Accumulator[Any]


@dataclass
class LocalGroupBy(Operation):
    extractor: optype.Extractor
    reducer: optype.Reducer[Any]


@dataclass
class Limit(Operation):
    length: int
    start: int


@dataclass
class Collect(Operation):
    pass


@dataclass
class Repartition(Operation):
    extractor: optype.Extractor


@dataclass
class Aggregate(Operation):
    zero: Any
    seqOp: optype.Accumulator[Any]
    combOp: optype.Accumulator[Any]


@dataclass
class AggregateBy(Operation):
    extractor: optype.Extractor
    zero: Any
    seqOp: optype.Reducer[Any]
    combOp: optype.Reducer[Any]


@dataclass
class GroupBy(Operation):
    extractor: optype.Extractor
    reducer: optype.Reducer[Any]


@dataclass
class BatchGroupBy(Operation):
    extractor: optype.Extractor
    reducer: optype.BatchReducer[Any]


@dataclass
class Sort(Operation):
    reverse: bool = True


@dataclass
class Distinct(Operation):
    pass


@dataclass
class Count(Operation):
    pass


@dataclass
class CountBy(Operation):
    extractor: optype.Extractor


@dataclass
class Avg(Operation):
    extractor: optype.Extractor



from typing import Any, Callable, Dict, Iterable, Type, TypeVar, Union

# Type aliases for type hints

T = TypeVar("T")
Key = TypeVar("Key")
Val = TypeVar("Val")

Constructor = Union[Type[T], Callable[[Any], T]]

RedisType = Union[bytes, int, float]
SafeType = Union[bytes, int, float, str]
SupportedType = Union[bool, str, bytes, int, float]

RedisKey = Union[str, bytes]

Record = dict
Callback = Callable[[Record], Any]
Extractor = Callable[[Record], T]
Mapper = Callable[[Record], Record]
Expander = Callable[[Record], Iterable[Dict[str, Any]]]
Processor = Callable[[Record], None]
Filterer = Callable[[Record], bool]
Accumulator = Callable[[T, Record], T]
Reducer = Callable[[RedisKey, T, Record], T]
BatchReducer = Callable[[RedisKey, Iterable[Dict[str, Any]]], T]
